 /* BonkEnc Audio Encoder
  * Copyright (C) 2001-2008 Robert Kausch <robert.kausch@bonkenc.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the "GNU General Public License".
  *
  * THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE. */

#include <input/filter-in-lame.h>

#include <dllinterfaces.h>

BonkEnc::FilterInLAME::FilterInLAME(Config *config, Track *format) : InputFilter(config, format)
{
	packageSize = 0;
}

BonkEnc::FilterInLAME::~FilterInLAME()
{
}

Bool BonkEnc::FilterInLAME::Activate()
{
	ex_lame_decode_init();

	InStream	*f_in = new InStream(STREAM_DRIVER, driver);

	SkipID3v2Tag(f_in);

	delete f_in;

	return true;
}

Bool BonkEnc::FilterInLAME::Deactivate()
{
	ex_lame_decode_exit();

	return true;
}

Int BonkEnc::FilterInLAME::ReadData(Buffer<UnsignedByte> &data, Int size)
{
	if (size <= 0) return -1;

	inBytes += size;

	data.Resize(size);

	size = driver->ReadData(data, size);

	pcm_l.Resize(size * 64);
	pcm_r.Resize(size * 64);

	Int	 nSamples = ex_lame_decode(data, size, pcm_l, pcm_r);

	data.Resize(nSamples * format->channels * (format->bits / 8));

	for (Int i = 0; i < nSamples; i++)
	{
		for (Int j = 0; j < format->channels; j++) ((short *) (unsigned char *) data)[format->channels * i + j] = (j == 0) ? pcm_l[i] : pcm_r[i];
	}

	return data.Size();
}

Bool BonkEnc::FilterInLAME::SkipID3v2Tag(InStream *in)
{
	/* Check for an ID3v2 tag at the beginning of the
	 * file and skip it if it exists as LAME may crash
	 * on unsynchronized tags.
	 */
	if (in->InputString(3) == "ID3")
	{
		in->InputNumber(2); // ID3 version
		in->InputNumber(1); // Flags

		/* Read tag size as a 4 byte unsynchronized integer.
		 */
		Int	 tagSize = (in->InputNumber(1) << 21) +
				   (in->InputNumber(1) << 14) +
				   (in->InputNumber(1) <<  7) +
				   (in->InputNumber(1)      );

		in->RelSeek(tagSize);

		inBytes += (tagSize + 10);
	}
	else
	{
		in->Seek(0);
	}

	return True;
}

BonkEnc::Track *BonkEnc::FilterInLAME::GetFileInfo(const String &inFile)
{
	ex_lame_decode_init();

	Track		*nFormat = new Track;
	InStream	*f_in = new InStream(STREAM_FILE, inFile, IS_READONLY);

	nFormat->order		= BYTE_INTEL;
	nFormat->bits		= 16;
	nFormat->fileSize	= f_in->Size();
	nFormat->length		= -1;

	SkipID3v2Tag(f_in);

	Buffer<unsigned char>	 buffer(4096);

	do
	{
		f_in->InputData((void *) buffer, buffer.Size());

		pcm_l.Resize(buffer.Size() * 64);
		pcm_r.Resize(buffer.Size() * 64);

		mp3data_struct	 mp3data;

		ZeroMemory(&mp3data, sizeof(mp3data));

		Int	 nSamples = ex_lame_decode_headers(buffer, buffer.Size(), pcm_l, pcm_r, &mp3data);

		if (mp3data.header_parsed && nSamples > mp3data.framesize)
		{
			nFormat->channels	= mp3data.stereo;
			nFormat->rate		= mp3data.samplerate;

			if (mp3data.nsamp > 0)		nFormat->length = mp3data.nsamp * nFormat->channels;
			else if (mp3data.bitrate > 0)	nFormat->approxLength = nFormat->fileSize / (mp3data.bitrate * 1000 / 8) * nFormat->rate * nFormat->channels;

			break;
		}
	}
	while (f_in->GetPos() < f_in->Size());

	delete f_in;

	ex_lame_decode_exit();

	if (currentConfig->enable_id3)
	{
		nFormat->track = -1;
		nFormat->outfile = NIL;

		ParseID3V2Tag(inFile, nFormat);
	}

	return nFormat;
}
